Package: tidyllm
Title: Tidy Integration of Large Language Models
Version: 0.5.0
Authors@R: c(
    person("Eduard", "Brüll", NULL, "eduard.bruell@zew.de", role = c("aut", "cre")),
    person("Jia", "Zhang", NULL, NULL, role = "ctb")
  )
Description: A tidy interface for integrating large language model (LLM) APIs such as 'Claude', 'OpenAI', 'Gemini', 'Mistral', and local models via 'Ollama' into R workflows. The package supports text, image, audio, video, and document interactions; a unified media interface for attaching inline files or uploading to provider file stores; batch request APIs for cost-efficient large-scale processing; and a pipeline-oriented interface for seamless integration into data workflows. Web services are available at <https://www.anthropic.com>, <https://openai.com>, <https://aistudio.google.com/>, <https://mistral.ai/> and <https://ollama.com>.
License: MIT + file LICENSE
URL: https://edubruell.github.io/tidyllm/
BugReports: https://github.com/edubruell/tidyllm/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), tidyverse, httptest2,
        httpuv, ellmer
Imports: S7 (>= 0.2.0), base64enc, glue, jsonlite, curl, httr2 (>=
        1.1.1), lubridate, purrr, rlang, stringr, grDevices, pdftools,
        tibble, cli, png, lifecycle
Collate: 'tidyllm-package.R' 'utilites.R' 'embedding_helpers.R'
        'LLMMessage.R' 'APIProvider.R' 'llm_message.R' 'llm_verbs.R'
        'media.R' 'message_retrieval.R' 'perform_api_requests.R'
        'rate_limits.R' 'tidyllm_schema.R' 'tools.R'
        'api_chat_completions.R' 'api_openai.R' 'api_azure_openai.R'
        'api_groq.R' 'api_mistral.R' 'api_perplexity.R'
        'api_deepseek.R' 'api_voyage.R' 'api_ollama.R' 'api_claude.R'
        'api_gemini.R' 'api_ellmer.R' 'api_openrouter.R'
        'api_llamacpp.R' 'pdfbatch.R' 'zzz.R'
Depends: R (>= 4.2.0)
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-29 20:30:12 UTC; ebr
Author: Eduard Brüll [aut, cre],
  Jia Zhang [ctb]
Maintainer: Eduard Brüll <eduard.bruell@zew.de>
Repository: CRAN
Date/Publication: 2026-04-30 08:30:08 UTC
Built: R 4.6.0; ; 2026-05-01 21:42:57 UTC; unix
