# Generated by roxygen2: do not edit by hand

export(.onLoad)
export(LLMMessage)
export(audio_file)
export(azure_openai)
export(azure_openai_chat)
export(azure_openai_embedding)
export(cancel_openai_batch)
export(chat)
export(chat_completions)
export(chat_completions_chat)
export(chat_ellmer)
export(chatgpt)
export(check_azure_openai_batch)
export(check_batch)
export(check_claude_batch)
export(check_gemini_batch)
export(check_groq_batch)
export(check_job)
export(check_mistral_batch)
export(check_openai_batch)
export(claude)
export(claude_chat)
export(claude_delete_file)
export(claude_file_metadata)
export(claude_list_files)
export(claude_list_models)
export(claude_upload_file)
export(claude_websearch)
export(deep_research)
export(deepseek)
export(deepseek_chat)
export(delete_file)
export(df_llm_message)
export(ellmer)
export(ellmer_tool)
export(embed)
export(fetch_azure_openai_batch)
export(fetch_batch)
export(fetch_claude_batch)
export(fetch_gemini_batch)
export(fetch_groq_batch)
export(fetch_job)
export(fetch_mistral_batch)
export(fetch_openai_batch)
export(field_chr)
export(field_dbl)
export(field_fct)
export(field_lgl)
export(field_object)
export(file_info)
export(gemini)
export(gemini_chat)
export(gemini_delete_file)
export(gemini_embedding)
export(gemini_file_metadata)
export(gemini_list_files)
export(gemini_list_models)
export(gemini_upload_file)
export(get_logprobs)
export(get_metadata)
export(get_reply)
export(get_reply_data)
export(get_user_message)
export(groq)
export(groq_chat)
export(groq_list_models)
export(groq_transcribe)
export(img)
export(last_metadata)
export(last_reply)
export(last_reply_data)
export(last_user_message)
export(list_azure_openai_batches)
export(list_batches)
export(list_claude_batches)
export(list_files)
export(list_gemini_batches)
export(list_groq_batches)
export(list_hf_gguf_files)
export(list_mistral_batches)
export(list_models)
export(list_openai_batches)
export(llamacpp)
export(llamacpp_chat)
export(llamacpp_delete_model)
export(llamacpp_download_model)
export(llamacpp_embedding)
export(llamacpp_health)
export(llamacpp_list_local_models)
export(llamacpp_list_models)
export(llamacpp_rerank)
export(llm_message)
export(mistral)
export(mistral_chat)
export(mistral_embedding)
export(mistral_list_models)
export(ollama)
export(ollama_chat)
export(ollama_delete_model)
export(ollama_download_model)
export(ollama_embedding)
export(ollama_list_models)
export(openai)
export(openai_chat)
export(openai_check_research)
export(openai_code_interpreter)
export(openai_deep_research)
export(openai_embedding)
export(openai_fetch_research)
export(openai_list_models)
export(openai_websearch)
export(openrouter)
export(openrouter_chat)
export(openrouter_credits)
export(openrouter_embedding)
export(openrouter_generation)
export(openrouter_list_models)
export(pdf_file)
export(pdf_page_batch)
export(perplexity)
export(perplexity_chat)
export(perplexity_check_research)
export(perplexity_deep_research)
export(perplexity_fetch_research)
export(rate_limit_info)
export(send_azure_openai_batch)
export(send_batch)
export(send_claude_batch)
export(send_gemini_batch)
export(send_groq_batch)
export(send_mistral_batch)
export(send_ollama_batch)
export(send_openai_batch)
export(tidyllm_schema)
export(tidyllm_tool)
export(upload_file)
export(video_file)
export(voyage)
export(voyage_embedding)
export(voyage_rerank)
if (getRversion() < "4.3.0") importFrom("S7", "@")
import(S7)
import(base64enc)
import(pdftools)
importFrom(base64enc,base64encode)
importFrom(glue,glue)
importFrom(grDevices,dev.copy)
importFrom(grDevices,dev.off)
importFrom(grDevices,png)
importFrom(jsonlite,toJSON)
importFrom(rlang,"%||%")
importFrom(tibble,as_tibble)
