structure(list(method = "POST", url = "https://api.perplexity.ai/chat/completions", 
    status_code = 200L, headers = structure(list(date = "Thu, 21 Aug 2025 12:41:56 GMT", 
        `content-type` = "application/json", `content-encoding` = "gzip", 
        `cf-cache-status` = "DYNAMIC", `set-cookie` = "REDACTED", 
        `strict-transport-security` = "max-age=15552000; includeSubDomains; preload", 
        server = "cloudflare", `cf-ray` = "972a350c8e486937-FRA"), redact = character(0), class = "httr2_headers"), 
    body = charToRaw("{\"id\": \"d632bdec-4fa8-4674-abfe-e181a736fa62\", \"model\": \"sonar\", \"created\": 1755780116, \"usage\": {\"prompt_tokens\": 8, \"completion_tokens\": 19, \"total_tokens\": 27, \"search_context_size\": \"low\", \"cost\": {\"input_tokens_cost\": 0.0, \"output_tokens_cost\": 0.0, \"request_cost\": 0.005, \"total_cost\": 0.005}}, \"citations\": [\"https://www.m1-project.com/blog/what-is-perplexity-ai-and-how-it-works\", \"https://web.stanford.edu/class/archive/cs/cs224n/cs224n.1224/posters/116768967.pdf\", \"https://www.techtarget.com/searchenterpriseai/definition/Perplexity-AI\", \"https://www.poetrysoup.com/poems/perplexity\", \"https://www.digitalocean.com/resources/articles/what-is-perplexity-ai\"], \"search_results\": [{\"title\": \"What Is Perplexity AI and How It Works - M1-Project\", \"url\": \"https://www.m1-project.com/blog/what-is-perplexity-ai-and-how-it-works\", \"date\": \"2025-07-24\", \"last_updated\": \"2025-07-24\"}, {\"title\": \"Haiku Generation models\", \"url\": \"https://web.stanford.edu/class/archive/cs/cs224n/cs224n.1224/posters/116768967.pdf\", \"date\": null, \"last_updated\": null}, {\"title\": \"What is Perplexity AI? | Definition from TechTarget\", \"url\": \"https://www.techtarget.com/searchenterpriseai/definition/Perplexity-AI\", \"date\": \"2025-03-04\", \"last_updated\": \"2025-08-14\"}, {\"title\": \"Examples of Poems about Perplexity\", \"url\": \"https://www.poetrysoup.com/poems/perplexity\", \"date\": \"2020-07-27\", \"last_updated\": \"2025-08-04\"}, {\"title\": \"What is Perplexity AI? A Smarter Way to Search | DigitalOcean\", \"url\": \"https://www.digitalocean.com/resources/articles/what-is-perplexity-ai\", \"date\": \"2025-03-13\", \"last_updated\": \"2025-08-01\"}], \"object\": \"chat.completion\", \"choices\": [{\"index\": 0, \"finish_reason\": \"stop\", \"message\": {\"role\": \"assistant\", \"content\": \"Silent web unfolds,  \\nPerplexity finds the truth\\u2014  \\nAnswers bloom like spring.\"}, \"delta\": {\"role\": \"assistant\", \"content\": \"\"}}]}"), 
    cache = new.env(parent = emptyenv())), class = "httr2_response")
