structure(list(method = "POST", url = "https://api.openai.com/v1/chat/completions", 
    status_code = 200L, headers = structure(list(date = "Sun, 16 Nov 2025 13:05:38 GMT", 
        `content-type` = "application/json", `access-control-expose-headers` = "X-Request-ID", 
        `openai-organization` = "user-j2dlcmauhfkzofrqjehceoc7", 
        `openai-processing-ms` = "1099", `openai-project` = "proj_wN2FU9W9VUckqc7nz44xte9H", 
        `openai-version` = "2020-10-01", `x-envoy-upstream-service-time` = "1313", 
        `x-ratelimit-limit-requests` = "5000", `x-ratelimit-limit-tokens` = "800000", 
        `x-ratelimit-remaining-requests` = "4999", `x-ratelimit-remaining-tokens` = "799976", 
        `x-ratelimit-reset-requests` = "12ms", `x-ratelimit-reset-tokens` = "1ms", 
        `x-request-id` = "req_ea0f18fede984ed0bb949aa79be4c3b6", 
        `x-openai-proxy-wasm` = "v0.1", `cf-cache-status` = "DYNAMIC", 
        `set-cookie` = "REDACTED", `strict-transport-security` = "max-age=31536000; includeSubDomains; preload", 
        `x-content-type-options` = "nosniff", `set-cookie` = "REDACTED", 
        server = "cloudflare", `cf-ray` = "99f7336e2c45dc7d-FRA", 
        `content-encoding` = "gzip", `alt-svc` = "h3=\":443\"; ma=86400"), redact = character(0), class = "httr2_headers"), 
    body = charToRaw("{\n  \"id\": \"chatcmpl-CcWjhJ3qP05CLu9YpiImmkM8ZlYjs\",\n  \"object\": \"chat.completion\",\n  \"created\": 1763298337,\n  \"model\": \"gpt-5.1-chat-latest\",\n  \"choices\": [\n    {\n      \"index\": 0,\n      \"message\": {\n        \"role\": \"assistant\",\n        \"content\": \"{\\\"area\\\":42}\",\n        \"refusal\": null,\n        \"annotations\": []\n      },\n      \"finish_reason\": \"stop\"\n    }\n  ],\n  \"usage\": {\n    \"prompt_tokens\": 53,\n    \"completion_tokens\": 18,\n    \"total_tokens\": 71,\n    \"prompt_tokens_details\": {\n      \"cached_tokens\": 0,\n      \"audio_tokens\": 0\n    },\n    \"completion_tokens_details\": {\n      \"reasoning_tokens\": 0,\n      \"audio_tokens\": 0,\n      \"accepted_prediction_tokens\": 0,\n      \"rejected_prediction_tokens\": 0\n    }\n  },\n  \"service_tier\": \"default\",\n  \"system_fingerprint\": null\n}\n"), 
    cache = new.env(parent = emptyenv())), class = "httr2_response")
