#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom grDevices dev.copy
#' @importFrom grDevices dev.off
#' @importFrom grDevices png
#' @importFrom tibble as_tibble
#' @import S7
## usethis namespace: end
NULL
