structure(list(method = "POST", url = "https://api.mistral.ai/v1/chat/completions", 
    status_code = 200L, headers = structure(list(date = "Thu, 21 Aug 2025 12:41:10 GMT", 
        `content-type` = "application/json", `content-encoding` = "gzip", 
        `mistral-correlation-id` = "0198cca5-ae6f-7c35-9c52-d3340710d226", 
        `x-kong-request-id` = "0198cca5-ae6f-7c35-9c52-d3340710d226", 
        `x-envoy-upstream-service-time` = "3080", `x-ratelimit-limit-tokens-minute` = "2000000", 
        `x-ratelimit-remaining-tokens-minute` = "1999932", `x-ratelimit-limit-tokens-month` = "10000000000", 
        `x-ratelimit-remaining-tokens-month` = "9999999932", 
        `x-ratelimit-tokens-query-cost` = "68", `x-ratelimit-limit-req-10-second` = "60", 
        `x-ratelimit-remaining-req-10-second` = "59", `access-control-allow-origin` = "*", 
        `x-kong-upstream-latency` = "3081", `x-kong-proxy-latency` = "5", 
        `cf-cache-status` = "DYNAMIC", `set-cookie` = "REDACTED", 
        `set-cookie` = "REDACTED", `strict-transport-security` = "max-age=15552000; includeSubDomains", 
        server = "cloudflare", `cf-ray` = "972a33ea0e9b691b-FRA", 
        `alt-svc` = "h3=\":443\"; ma=86400"), redact = character(0), class = "httr2_headers"), 
    body = charToRaw("{\"id\":\"d0a4a06db45446809c07ca9d1cc4158d\",\"created\":1755780066,\"model\":\"mistral-large-latest\",\"usage\":{\"prompt_tokens\":7,\"total_tokens\":68,\"completion_tokens\":61},\"object\":\"chat.completion\",\"choices\":[{\"index\":0,\"finish_reason\":\"stop\",\"message\":{\"role\":\"assistant\",\"tool_calls\":null,\"content\":\"Hello, world! 🌍✨ How can I brighten your day today? Whether you're coding, exploring, or just saying hi, I'm here to help! 😊\\n\\n(And if you're learning programming—great start with that classic greeting! 🚀)\"}}]}"), 
    cache = new.env(parent = emptyenv())), class = "httr2_response")
