structure(list(method = "POST", url = "https://api.anthropic.com/v1/messages", 
    status_code = 200L, headers = structure(list(date = "Thu, 21 Aug 2025 12:41:00 GMT", 
        `content-type` = "application/json", `content-encoding` = "gzip", 
        `anthropic-ratelimit-input-tokens-limit` = "80000", `anthropic-ratelimit-input-tokens-remaining` = "80000", 
        `anthropic-ratelimit-input-tokens-reset` = "2025-08-21T12:40:59Z", 
        `anthropic-ratelimit-output-tokens-limit` = "16000", 
        `anthropic-ratelimit-output-tokens-remaining` = "16000", 
        `anthropic-ratelimit-output-tokens-reset` = "2025-08-21T12:41:00Z", 
        `anthropic-ratelimit-requests-limit` = "1000", `anthropic-ratelimit-requests-remaining` = "999", 
        `anthropic-ratelimit-requests-reset` = "2025-08-21T12:40:59Z", 
        `anthropic-ratelimit-tokens-limit` = "96000", `anthropic-ratelimit-tokens-remaining` = "96000", 
        `anthropic-ratelimit-tokens-reset` = "2025-08-21T12:40:59Z", 
        `request-id` = "req_011CSLsW7mFqvHHnzrKdKjAE", `strict-transport-security` = "max-age=31536000; includeSubDomains; preload", 
        `anthropic-organization-id` = "36562039-1ae4-48e6-ae28-1e517e3d2d12", 
        `x-envoy-upstream-service-time` = "690", via = "1.1 google", 
        `cf-cache-status` = "DYNAMIC", `x-robots-tag` = "none", 
        server = "cloudflare", `cf-ray` = "972a33bb3977d246-FRA"), redact = character(0), class = "httr2_headers"), 
    body = charToRaw("{\"id\":\"msg_01QgNtCXZKCJgpWHW3NEwmdP\",\"type\":\"message\",\"role\":\"assistant\",\"model\":\"claude-3-5-sonnet-20240620\",\"content\":[{\"type\":\"text\",\"text\":\"Hello! How can I assist you today? Is there anything specific you'd like to know or discuss?\"}],\"stop_reason\":\"end_turn\",\"stop_sequence\":null,\"usage\":{\"input_tokens\":16,\"cache_creation_input_tokens\":0,\"cache_read_input_tokens\":0,\"cache_creation\":{\"ephemeral_5m_input_tokens\":0,\"ephemeral_1h_input_tokens\":0},\"output_tokens\":24,\"service_tier\":\"standard\"}}"), 
    cache = new.env(parent = emptyenv())), class = "httr2_response")
