structure(list(method = "GET", url = "https://api.openai.com/v1/batches?limit=20", 
    status_code = 200L, headers = structure(list(date = "Thu, 21 Aug 2025 12:41:49 GMT", 
        `content-type` = "application/json", `openai-version` = "2020-10-01", 
        `openai-organization` = "user-j2dlcmauhfkzofrqjehceoc7", 
        `openai-project` = "proj_wN2FU9W9VUckqc7nz44xte9H", `x-request-id` = "req_e2c3d7a1e53f39735069740eff16fe0e", 
        `openai-processing-ms` = "215", `x-envoy-upstream-service-time` = "217", 
        `strict-transport-security` = "max-age=31536000; includeSubDomains; preload", 
        `cf-cache-status` = "DYNAMIC", `set-cookie` = "REDACTED", 
        `x-content-type-options` = "nosniff", `set-cookie` = "REDACTED", 
        server = "cloudflare", `cf-ray` = "972a34efca31bbc2-FRA", 
        `content-encoding` = "gzip", `alt-svc` = "h3=\":443\"; ma=86400"), redact = character(0), class = "httr2_headers"), 
    body = charToRaw("{\n  \"object\": \"list\",\n  \"data\": [\n    {\n      \"id\": \"batch_680b8e3d92208190a7c2b25ef89cd8b5\",\n      \"object\": \"batch\",\n      \"endpoint\": \"/v1/chat/completions\",\n      \"errors\": null,\n      \"input_file_id\": \"file-8LjgrmGyywoGv8vrDb2mWb\",\n      \"completion_window\": \"24h\",\n      \"status\": \"completed\",\n      \"output_file_id\": \"file-D1Tn2JDcYztLWs1P59XUtx\",\n      \"error_file_id\": null,\n      \"created_at\": 1745587773,\n      \"in_progress_at\": 1745587775,\n      \"expires_at\": 1745674173,\n      \"finalizing_at\": 1745598803,\n      \"completed_at\": 1745598858,\n      \"failed_at\": null,\n      \"expired_at\": null,\n      \"cancelling_at\": null,\n      \"cancelled_at\": null,\n      \"request_counts\": {\n        \"total\": 575,\n        \"completed\": 575,\n        \"failed\": 0\n      },\n      \"metadata\": null\n    },\n    {\n      \"id\": \"batch_67df363bc4508190a0fb4160b4117949\",\n      \"object\": \"batch\",\n      \"endpoint\": \"/v1/chat/completions\",\n      \"errors\": null,\n      \"input_file_id\": \"file-J763TdywMfH4xVtc8gb1v5\",\n      \"completion_window\": \"24h\",\n      \"status\": \"completed\",\n      \"output_file_id\": \"file-5yu9BV9MPdneMkmHBN7Sfd\",\n      \"error_file_id\": null,\n      \"created_at\": 1742681659,\n      \"in_progress_at\": 1742681660,\n      \"expires_at\": 1742768059,\n      \"finalizing_at\": 1742681727,\n      \"completed_at\": 1742681730,\n      \"failed_at\": null,\n      \"expired_at\": null,\n      \"cancelling_at\": null,\n      \"cancelled_at\": null,\n      \"request_counts\": {\n        \"total\": 24,\n        \"completed\": 24,\n        \"failed\": 0\n      },\n      \"metadata\": null\n    },\n    {\n      \"id\": \"batch_67daa551c13c8190ab7b855832f423ca\",\n      \"object\": \"batch\",\n      \"endpoint\": \"/v1/chat/completions\",\n      \"errors\": null,\n      \"input_file_id\": \"file-Y1gaustRLWZW31bmgGPJHP\",\n      \"completion_window\": \"24h\",\n      \"status\": \"completed\",\n      \"output_file_id\": \"file-G8zouWYRcyxEvAi75yBE1u\",\n      \"error_file_id\": null,\n      \"created_at\": 1742382417,\n      \"in_progress_at\": 1742382418,\n      \"expires_at\": 1742468817,\n      \"finalizing_at\": 1742384829,\n      \"completed_at\": 1742384830,\n      \"failed_at\": null,\n      \"expired_at\": null,\n      \"cancelling_at\": null,\n      \"cancelled_at\": null,\n      \"request_counts\": {\n        \"total\": 4,\n        \"completed\": 4,\n        \"failed\": 0\n      },\n      \"metadata\": null\n    },\n    {\n      \"id\": \"batch_67daa459f6388190a343393776061bbe\",\n      \"object\": \"batch\",\n      \"endpoint\": \"/v1/chat/completions\",\n      \"errors\": null,\n      \"input_file_id\": \"file-A4amUoNcQz4aiB3MSuwCtx\",\n      \"completion_window\": \"24h\",\n      \"status\": \"completed\",\n      \"output_file_id\": \"file-JRMPb3f1v7wZWeMko7b5BZ\",\n      \"error_file_id\": null,\n      \"created_at\": 1742382170,\n      \"in_progress_at\": 1742382170,\n      \"expires_at\": 1742468570,\n      \"finalizing_at\": 1742382232,\n      \"completed_at\": 1742382232,\n      \"failed_at\": null,\n      \"expired_at\": null,\n      \"cancelling_at\": null,\n      \"cancelled_at\": null,\n      \"request_counts\": {\n        \"total\": 3,\n        \"completed\": 3,\n        \"failed\": 0\n      },\n      \"metadata\": null\n    },\n    {\n      \"id\": \"batch_67b9268b399c8190b134c840c184d160\",\n      \"object\": \"batch\",\n      \"endpoint\": \"/v1/chat/completions\",\n      \"errors\": null,\n      \"input_file_id\": \"file-2Z5UABm7zAUYPzBZP28SG1\",\n      \"completion_window\": \"24h\",\n      \"status\": \"completed\",\n      \"output_file_id\": \"file-RmWfqciD6d44ydhv1SUVrp\",\n      \"error_file_id\": null,\n      \"created_at\": 1740187275,\n      \"in_progress_at\": 1740187276,\n      \"expires_at\": 1740273675,\n      \"finalizing_at\": 1740187380,\n      \"completed_at\": 1740187382,\n      \"failed_at\": null,\n      \"expired_at\": null,\n      \"cancelling_at\": null,\n      \"cancelled_at\": null,\n      \"request_counts\": {\n        \"total\": 3,\n        \"completed\": 3,\n        \"failed\": 0\n      },\n      \"metadata\": null\n    },\n    {\n      \"id\": \"batch_67b66cd8678481909ce499abded372c6\",\n      \"object\": \"batch\",\n      \"endpoint\": \"/v1/chat/completions\",\n      \"errors\": null,\n      \"input_file_id\": \"file-Rr9bwuBaxKFSs5uVj9eZtT\",\n      \"completion_window\": \"24h\",\n      \"status\": \"completed\",\n      \"output_file_id\": \"file-KiBzbjkA1rhopE8cQh9FtY\",\n      \"error_file_id\": null,\n      \"created_at\": 1740008664,\n      \"in_progress_at\": 1740008664,\n      \"expires_at\": 1740095064,\n      \"finalizing_at\": 1740009030,\n      \"completed_at\": 1740009031,\n      \"failed_at\": null,\n      \"expired_at\": null,\n      \"cancelling_at\": null,\n      \"cancelled_at\": null,\n      \"request_counts\": {\n        \"total\": 4,\n        \"completed\": 4,\n        \"failed\": 0\n      },\n      \"metadata\": null\n    },\n    {\n      \"id\": \"batch_674c7bf16aa4819080dfa5e9b8efbdee\",\n      \"object\": \"batch\",\n      \"endpoint\": \"/v1/chat/completions\",\n      \"errors\": null,\n      \"input_file_id\": \"file-HXCQ23DJLvooBEeV6RCpxk\",\n      \"completion_window\": \"24h\",\n      \"status\": \"completed\",\n      \"output_file_id\": \"file-W2HSoUBnPfZZWdovVut7wC\",\n      \"error_file_id\": null,\n      \"created_at\": 1733065713,\n      \"in_progress_at\": 1733065713,\n      \"expires_at\": 1733152113,\n      \"finalizing_at\": 1733065794,\n      \"completed_at\": 1733065796,\n      \"failed_at\": null,\n      \"expired_at\": null,\n      \"cancelling_at\": null,\n      \"cancelled_at\": null,\n      \"request_counts\": {\n        \"total\": 31,\n        \"completed\": 31,\n        \"failed\": 0\n      },\n      \"metadata\": null\n    },\n    {\n      \"id\": \"batch_674b399f2e288190a9aad4d6d2b13587\",\n      \"object\": \"batch\",\n      \"endpoint\": \"/v1/chat/completions\",\n      \"errors\": null,\n      \"input_file_id\": \"file-548TC1XrHkJH6SPyNd3RHk\",\n      \"completion_window\": \"24h\",\n      \"status\": \"completed\",\n      \"output_file_id\": \"file-JcZ57aJ1SMQPdwZwjonQTT\",\n      \"error_file_id\": null,\n      \"created_at\": 1732983199,\n      \"in_progress_at\": 1732983199,\n      \"expires_at\": 1733069599,\n      \"finalizing_at\": 1732983234,\n      \"completed_at\": 1732983234,\n      \"failed_at\": null,\n      \"expired_at\": null,\n      \"cancelling_at\": null,\n      \"cancelled_at\": null,\n      \"request_counts\": {\n        \"total\": 3,\n        \"completed\": 3,\n        \"failed\": 0\n      },\n      \"metadata\": null\n    },\n    {\n      \"id\": \"batch_674b186550e8819087b2407dbc200ef7\",\n      \"object\": \"batch\",\n      \"endpoint\": \"/v1/chat/completions\",\n      \"errors\": null,\n      \"input_file_id\": \"file-2k454PBUtkhmg1XT9FtzA1\",\n      \"completion_window\": \"24h\",\n      \"status\": \"completed\",\n      \"output_file_id\": \"file-CzwinJRHXjg3Cbec8KMDKd\",\n      \"error_file_id\": null,\n      \"created_at\": 1732974693,\n      \"in_progress_at\": 1732974694,\n      \"expires_at\": 1733061093,\n      \"finalizing_at\": 1732974706,\n      \"completed_at\": 1732974707,\n      \"failed_at\": null,\n      \"expired_at\": null,\n      \"cancelling_at\": null,\n      \"cancelled_at\": null,\n      \"request_counts\": {\n        \"total\": 3,\n        \"completed\": 3,\n        \"failed\": 0\n      },\n      \"metadata\": null\n    },\n    {\n      \"id\": \"batch_674b17e882f481909f438bec763b0375\",\n      \"object\": \"batch\",\n      \"endpoint\": \"/v1/chat/completions\",\n      \"errors\": null,\n      \"input_file_id\": \"file-HXuoMmZTYCDcB19bTDYcNC\",\n      \"completion_window\": \"24h\",\n      \"status\": \"completed\",\n      \"output_file_id\": \"file-X3j7WdUh5YyFxJ13WtobqB\",\n      \"error_file_id\": null,\n      \"created_at\": 1732974568,\n      \"in_progress_at\": 1732974569,\n      \"expires_at\": 1733060968,\n      \"finalizing_at\": 1732974578,\n      \"completed_at\": 1732974579,\n      \"failed_at\": null,\n      \"expired_at\": null,\n      \"cancelling_at\": null,\n      \"cancelled_at\": null,\n      \"request_counts\": {\n        \"total\": 3,\n        \"completed\": 3,\n        \"failed\": 0\n      },\n      \"metadata\": null\n    },\n    {\n      \"id\": \"batch_673f45f04ed88190b54425d1ad02097d\",\n      \"object\": \"batch\",\n      \"endpoint\": \"/v1/chat/completions\",\n      \"errors\": null,\n      \"input_file_id\": \"file-lbYKD1lAVtqELKXv57YeyQIx\",\n      \"completion_window\": \"24h\",\n      \"status\": \"completed\",\n      \"output_file_id\": \"file-Vf06YkflOrr09vd1fbaSg8l9\",\n      \"error_file_id\": null,\n      \"created_at\": 1732199920,\n      \"in_progress_at\": 1732199921,\n      \"expires_at\": 1732286320,\n      \"finalizing_at\": 1732201331,\n      \"completed_at\": 1732201335,\n      \"failed_at\": null,\n      \"expired_at\": null,\n      \"cancelling_at\": null,\n      \"cancelled_at\": null,\n      \"request_counts\": {\n        \"total\": 24,\n        \"completed\": 24,\n        \"failed\": 0\n      },\n      \"metadata\": null\n    },\n    {\n      \"id\": \"batch_673df61b7fa881909a3636f92a8edbba\",\n      \"object\": \"batch\",\n      \"endpoint\": \"/v1/chat/completions\",\n      \"errors\": null,\n      \"input_file_id\": \"file-x6a5otNxlsHZJae5N5UOd0lW\",\n      \"completion_window\": \"24h\",\n      \"status\": \"completed\",\n      \"output_file_id\": \"file-SNR0fMQAFJlT5CTCz1VoM3fQ\",\n      \"error_file_id\": null,\n      \"created_at\": 1732113947,\n      \"in_progress_at\": 1732113948,\n      \"expires_at\": 1732200347,\n      \"finalizing_at\": 1732113975,\n      \"completed_at\": 1732113976,\n      \"failed_at\": null,\n      \"expired_at\": null,\n      \"cancelling_at\": null,\n      \"cancelled_at\": null,\n      \"request_counts\": {\n        \"total\": 3,\n        \"completed\": 3,\n        \"failed\": 0\n      },\n      \"metadata\": null\n    },\n    {\n      \"id\": \"batch_67336038ab588190a73c21909032d51a\",\n      \"object\": \"batch\",\n      \"endpoint\": \"/v1/chat/completions\",\n      \"errors\": null,\n      \"input_file_id\": \"file-T4kerjYXuZNyHaiFauZt5xBF\",\n      \"completion_window\": \"24h\",\n      \"status\": \"completed\",\n      \"output_file_id\": \"file-DbEjAkYOZjADNKBytBroXtNL\",\n      \"error_file_id\": null,\n      \"created_at\": 1731420216,\n      \"in_progress_at\": 1731420217,\n      \"expires_at\": 1731506616,\n      \"finalizing_at\": 1731420433,\n      \"completed_at\": 1731420434,\n      \"failed_at\": null,\n      \"expired_at\": null,\n      \"cancelling_at\": null,\n      \"cancelled_at\": null,\n      \"request_counts\": {\n        \"total\": 2,\n        \"completed\": 2,\n        \"failed\": 0\n      },\n      \"metadata\": null\n    },\n    {\n      \"id\": \"batch_67335c97baa881908914084a447a75a8\",\n      \"object\": \"batch\",\n      \"endpoint\": \"/v1/chat/completions\",\n      \"errors\": null,\n      \"input_file_id\": \"file-JnzHT7Dg4upZpstY2diCxkbq\",\n      \"completion_window\": \"24h\",\n      \"status\": \"completed\",\n      \"output_file_id\": \"file-5D1wTdE8Z5WUAhNCVjPu25dg\",\n      \"error_file_id\": null,\n      \"created_at\": 1731419287,\n      \"in_progress_at\": 1731419288,\n      \"expires_at\": 1731505687,\n      \"finalizing_at\": 1731453625,\n      \"completed_at\": 1731453626,\n      \"failed_at\": null,\n      \"expired_at\": null,\n      \"cancelling_at\": null,\n      \"cancelled_at\": null,\n      \"request_counts\": {\n        \"total\": 2,\n        \"completed\": 2,\n        \"failed\": 0\n      },\n      \"metadata\": null\n    },\n    {\n      \"id\": \"batch_672e77b2dd588190b2f751653989e98e\",\n      \"object\": \"batch\",\n      \"endpoint\": \"/v1/chat/completions\",\n      \"errors\": null,\n      \"input_file_id\": \"file-REqzkDJPSzOD8iVlztZdjnjH\",\n      \"completion_window\": \"24h\",\n      \"status\": \"completed\",\n      \"output_file_id\": \"file-nK0YxSWaAlxd36JqMc98rOvn\",\n      \"error_file_id\": null,\n      \"created_at\": 1731098547,\n      \"in_progress_at\": 1731098547,\n      \"expires_at\": 1731184947,\n      \"finalizing_at\": 1731098552,\n      \"completed_at\": 1731098553,\n      \"failed_at\": null,\n      \"expired_at\": null,\n      \"cancelling_at\": null,\n      \"cancelled_at\": null,\n      \"request_counts\": {\n        \"total\": 2,\n        \"completed\": 2,\n        \"failed\": 0\n      },\n      \"metadata\": null\n    },\n    {\n      \"id\": \"batch_672e7751c0148190bd8a56a9e1a97859\",\n      \"object\": \"batch\",\n      \"endpoint\": \"/v1/chat/completions\",\n      \"errors\": null,\n      \"input_file_id\": \"file-lU6SesGKEgH13pAivF30orc6\",\n      \"completion_window\": \"24h\",\n      \"status\": \"completed\",\n      \"output_file_id\": \"file-yD2R2GbVBEEYIR3CRb1EajXk\",\n      \"error_file_id\": null,\n      \"created_at\": 1731098449,\n      \"in_progress_at\": 1731098450,\n      \"expires_at\": 1731184849,\n      \"finalizing_at\": 1731098455,\n      \"completed_at\": 1731098456,\n      \"failed_at\": null,\n      \"expired_at\": null,\n      \"cancelling_at\": null,\n      \"cancelled_at\": null,\n      \"request_counts\": {\n        \"total\": 2,\n        \"completed\": 2,\n        \"failed\": 0\n      },\n      \"metadata\": null\n    },\n    {\n      \"id\": \"batch_6726c748bd588190a506a1c1882a3741\",\n      \"object\": \"batch\",\n      \"endpoint\": \"/v1/chat/completions\",\n      \"errors\": null,\n      \"input_file_id\": \"file-3JEyZLqIX0qj9o2pY96MpHNv\",\n      \"completion_window\": \"24h\",\n      \"status\": \"completed\",\n      \"output_file_id\": \"file-bbCv9zivkbf1ulnjf15MjVHx\",\n      \"error_file_id\": null,\n      \"created_at\": 1730594632,\n      \"in_progress_at\": 1730594634,\n      \"expires_at\": 1730681032,\n      \"finalizing_at\": 1730594681,\n      \"completed_at\": 1730594683,\n      \"failed_at\": null,\n      \"expired_at\": null,\n      \"cancelling_at\": null,\n      \"cancelled_at\": null,\n      \"request_counts\": {\n        \"total\": 24,\n        \"completed\": 24,\n        \"failed\": 0\n      },\n      \"metadata\": null\n    },\n    {\n      \"id\": \"batch_672671ea162c8190993e5871d9226fa0\",\n      \"object\": \"batch\",\n      \"endpoint\": \"/v1/chat/completions\",\n      \"errors\": null,\n      \"input_file_id\": \"file-6ECf6nJBp1NzglmoFQ1NXet5\",\n      \"completion_window\": \"24h\",\n      \"status\": \"completed\",\n      \"output_file_id\": \"file-zARt37HQsMaPOqeGytw6G24M\",\n      \"error_file_id\": null,\n      \"created_at\": 1730572778,\n      \"in_progress_at\": 1730572839,\n      \"expires_at\": 1730659178,\n      \"finalizing_at\": 1730573447,\n      \"completed_at\": 1730573448,\n      \"failed_at\": null,\n      \"expired_at\": null,\n      \"cancelling_at\": null,\n      \"cancelled_at\": null,\n      \"request_counts\": {\n        \"total\": 4,\n        \"completed\": 4,\n        \"failed\": 0\n      },\n      \"metadata\": null\n    },\n    {\n      \"id\": \"batch_67265e33b0148190b48d9d0c11f04ae1\",\n      \"object\": \"batch\",\n      \"endpoint\": \"/v1/chat/completions\",\n      \"errors\": null,\n      \"input_file_id\": \"file-bVodIkZMrEUuw8FOK4NmHQkn\",\n      \"completion_window\": \"24h\",\n      \"status\": \"completed\",\n      \"output_file_id\": null,\n      \"error_file_id\": \"file-NwRZbUZtQHlaEC81WkTulhc2\",\n      \"created_at\": 1730567731,\n      \"in_progress_at\": 1730567732,\n      \"expires_at\": 1730654131,\n      \"finalizing_at\": 1730567736,\n      \"completed_at\": 1730567737,\n      \"failed_at\": null,\n      \"expired_at\": null,\n      \"cancelling_at\": null,\n      \"cancelled_at\": null,\n      \"request_counts\": {\n        \"total\": 4,\n        \"completed\": 0,\n        \"failed\": 4\n      },\n      \"metadata\": null\n    },\n    {\n      \"id\": \"batch_67265d847bd48190afc0006531008ae4\",\n      \"object\": \"batch\",\n      \"endpoint\": \"/v1/chat/completions\",\n      \"errors\": null,\n      \"input_file_id\": \"file-io9wSihisrt4jqY6eWdnBWXs\",\n      \"completion_window\": \"24h\",\n      \"status\": \"completed\",\n      \"output_file_id\": \"file-AOOihugly3xB4S79wEZ0jTzw\",\n      \"error_file_id\": null,\n      \"created_at\": 1730567556,\n      \"in_progress_at\": 1730567557,\n      \"expires_at\": 1730653956,\n      \"finalizing_at\": 1730567560,\n      \"completed_at\": 1730567561,\n      \"failed_at\": null,\n      \"expired_at\": null,\n      \"cancelling_at\": null,\n      \"cancelled_at\": null,\n      \"request_counts\": {\n        \"total\": 2,\n        \"completed\": 2,\n        \"failed\": 0\n      },\n      \"metadata\": null\n    }\n  ],\n  \"first_id\": \"batch_680b8e3d92208190a7c2b25ef89cd8b5\",\n  \"last_id\": \"batch_67265d847bd48190afc0006531008ae4\",\n  \"has_more\": true\n}"), 
    cache = new.env(parent = emptyenv())), class = "httr2_response")
