% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api_gemini.R
\name{gemini_upload_file}
\alias{gemini_upload_file}
\title{Upload a File to Gemini API}
\usage{
gemini_upload_file(.file_path)
}
\arguments{
\item{.file_path}{The local file path of the file to upload.}
}
\value{
A tibble containing metadata about the uploaded file, including its name, URI, and MIME type.
}
\description{
Uploads a file to the Gemini API and returns its metadata as a tibble.
}
