% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/media.R
\name{audio_file}
\alias{audio_file}
\title{Create an Audio Object}
\usage{
audio_file(.path)
}
\arguments{
\item{.path}{The path to the audio file on disk.}
}
\value{
A \code{tidyllm_audio} object.
}
\description{
Stores a reference to a local audio file for use in multimodal messages.
The file is not encoded until the message is formatted for a provider.
}
