% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api_openai.R
\name{openai_code_interpreter}
\alias{openai_code_interpreter}
\title{OpenAI built-in code interpreter tool (server-executed)}
\usage{
openai_code_interpreter()
}
\value{
A TOOL object declaring the code_interpreter built-in.
}
\description{
OpenAI built-in code interpreter tool (server-executed)
}
