% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api_openai.R
\name{openai_check_research}
\alias{openai_check_research}
\title{Check the Status of an OpenAI Background Research Job}
\usage{
openai_check_research(.job, .max_tries = 3)
}
\arguments{
\item{.job}{A \code{tidyllm_research_job} object returned by \code{openai_deep_research(.background = TRUE)}.}

\item{.max_tries}{Maximum retries per HTTP request (default: \code{3}).}
}
\value{
An updated \code{tidyllm_research_job} with \verb{$status} set. If completed, \verb{$response}
is also populated and ready for \code{fetch_job()}.
}
\description{
Polls the status of an OpenAI background response created by
\code{openai_deep_research(.background = TRUE)}.
}
