% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/llm_verbs.R
\name{check_job}
\alias{check_job}
\title{Check the Status of a Batch or Research Job}
\usage{
check_job(.job, ...)
}
\arguments{
\item{.job}{An object with a \code{batch_id} attribute (from \code{send_batch()}) or
a \code{tidyllm_research_job} (from \code{deep_research(.background = TRUE)}).}

\item{...}{Additional arguments passed to the underlying function.}
}
\value{
Status information; type depends on \code{.job} class.
}
\description{
\code{check_job()} dispatches to \code{check_batch()} for batch objects or
\code{perplexity_check_research()} for \code{tidyllm_research_job} objects.
}
