% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api_openai.R
\name{openai_websearch}
\alias{openai_websearch}
\title{OpenAI built-in web search tool (server-executed)}
\usage{
openai_websearch()
}
\value{
A TOOL object declaring the web_search_preview built-in.
}
\description{
OpenAI built-in web search tool (server-executed)
}
