% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api_ellmer.R
\name{ellmer}
\alias{ellmer}
\title{Alias for the Ellmer Provider Function}
\usage{
ellmer(..., .called_from = NULL)
}
\arguments{
\item{...}{Additional parameters to pass through (for consistency with other providers).}

\item{.called_from}{An internal argument specifying which action (e.g.,
\code{chat}) the function is invoked from.}
}
\value{
A provider function that can be used with \code{tidyllm::chat()}.
}
\description{
The \code{chat_ellmer()} function acts as a provider interface for interacting with
ellmer chat objects through \code{tidyllm}'s  verb interface
}
\details{
This function allows you to use any ellmer chat object (e.g., from
\code{ellmer::chat_anthropic()}, \code{ellmer::chat_openai()}, etc.) as a stateless
backend for tidyllm. The ellmer object is cloned for each interaction to
maintain tidyllm's stateless approach.
}
