% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/llm_verbs.R
\name{upload_file}
\alias{upload_file}
\title{Upload a File to a Provider's File Store}
\usage{
upload_file(.provider, .path, ...)
}
\arguments{
\item{.provider}{A provider function call (e.g. \code{claude()}, \code{gemini()}, \code{openai()}).}

\item{.path}{Path to the local file to upload.}

\item{...}{Additional provider-specific arguments.}
}
\value{
A \code{tidyllm_file} object.
}
\description{
Upload a File to a Provider's File Store
}
