% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api_claude.R
\name{claude_websearch}
\alias{claude_websearch}
\title{Builtin Claude Web Search Tool}
\usage{
claude_websearch()
}
\description{
Returns a TOOL object for Claude's builtin web_search tool.
}
